/*
diccionario.c Wordle9511, ejemplo de juego sobrediseñado con TDAs.
Copyright (C) 2022 Sebastian Santisi <ssantisi@fi.uba.ar>
*/
#include "diccionario.h"

#include "config.h"
#include "utils.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Representa a un diccionario que tiene n palabras de longitud
// PALABRA_LONGITUD. Las palabras están finalizadas con '\0'.
// n == 0 <=> palabras == NULL.
struct diccionario {
    char (*palabras)[PALABRA_LONGITUD + 1]; // Ver vector.h
    size_t n;
};


diccionario_t *diccionario_crear(const char *fn) {
    FILE *f = fopen(fn, "rb");
    if(f == NULL)
        return NULL;

    diccionario_t *d = malloc(sizeof(diccionario_t));
    if(d == NULL) {
        fclose(f);
        return NULL;
    }

    d->palabras = NULL;
    d->n = 0;

    char palabra[PALABRA_LONGITUD + 1];
    palabra[PALABRA_LONGITUD] = '\0';
    while(fread(palabra, sizeof(char), PALABRA_LONGITUD, f) == PALABRA_LONGITUD) {
        char (*aux)[PALABRA_LONGITUD + 1] = realloc(d->palabras, (d->n + 1) * sizeof(char[PALABRA_LONGITUD + 1]));
        if(aux == NULL) {
            fclose(f);
            diccionario_destruir(d);
            return NULL;
        }
        d->palabras = aux;

        strcpy(d->palabras[d->n++], palabra);
    }

    fclose(f);

    return d;
}

void diccionario_destruir(diccionario_t *d) {
    free(d->palabras);
    free(d);
}

size_t diccionario_get_longitud(const diccionario_t *d) {
    return d->n;
}

bool diccionario_contiene(const diccionario_t *d, const char *palabra) {
    return bsearch(palabra, d->palabras, d->n, (PALABRA_LONGITUD + 1) * sizeof(char), (int (*)(const void *, const void *))strcmp) != NULL;
}

char *diccionario_obtener_palabra(const diccionario_t *d, size_t i) {
    if(i >= d->n)
        return NULL;

    return strdup(d->palabras[i]);
}
