/*
diccionario.h Wordle9511, ejemplo de juego sobrediseñado con TDAs.
Copyright (C) 2022 Sebastian Santisi <ssantisi@fi.uba.ar>
*/
#ifndef DICCIONARIO_H
#define DICCIONARIO_H

#include <stdbool.h>
#include <stddef.h>

// TDA que contiene un listado de palabras
typedef struct diccionario diccionario_t;

// Crea un diccionario desde el nombre fn de un archivo binario con las
// palabras.
// El archivo binario consiste en una sucesión
// a0 b0 c0 d0 e0 a1 b1 c1 d1 e1 a2 b2 c2 d2 e2 ...
// donde cada "abcde" representa una palabra.
// Las palabras del archivo deben estar ordenadas lexicográficamente y estar
// en mayúsculas.
diccionario_t *diccionario_crear(const char *fn);

// Libera la memoria asociada a d
void diccionario_destruir(diccionario_t *d);

// Getter de la longitud del diccionario d
size_t diccionario_get_longitud(const diccionario_t *d);

// Indica si la palabra está presente en d
bool diccionario_contiene(const diccionario_t *d, const char *palabra);

// Devuelve una copia dinámica de la palabra en el índice i de d
char *diccionario_obtener_palabra(const diccionario_t *d, size_t i);

#endif
