/*
interaccion.c Wordle9511, ejemplo de juego sobrediseñado con TDAs.
Copyright (C) 2022 Sebastian Santisi <ssantisi@fi.uba.ar>
*/
#include "interaccion.h"
#include "utils.h"
#include "ansi.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>


#define MAX_CADENA 100


// Estructura auxiliar para la tabla de búsqueda
struct colores {
    ansi_colores_t frente, fondo;
};

// Tabla de búsqueda de colores para un estado de letra
static const struct colores tabla_colores[] = {
    [LETRA_NO_ARRIESGADA] = {COLOR_NEGRO, COLOR_GRIS_CLARO},
    [LETRA_NO_ACERTADA] = {COLOR_BLANCO, COLOR_GRIS},
    [LETRA_POSICION_INCORRECTA] = {COLOR_BLANCO, COLOR_AMARILLO_OSCURO},
    [LETRA_POSICION_CORRECTA] = {COLOR_BLANCO, COLOR_VERDE_OSCURO},
};

// Tabla de búsqueda de traducción de letra_estado_t en cadena
// ACLARACIÓN: Esta tabla de búsqueda podría estar integrada a la anterior como
// un miembro adicional de la estructura.
static const char *tabla_ayudas[] = {
    [LETRA_NO_ARRIESGADA] = "Letra no arriesgada",
    [LETRA_NO_ACERTADA] = "Letra no acertada",
    [LETRA_POSICION_INCORRECTA] = "Letra en posición incorrecta",
    [LETRA_POSICION_CORRECTA] = "Letra en posición correcta",
};


char *leer_palabra(diccionario_t *d) {
    while(1) {
        char aux[MAX_CADENA];
        printf("Ingrese una palabra: ");
        if(fgets(aux, MAX_CADENA, stdin) == NULL)
            return NULL;

        for(size_t i = 0; aux[i]; i++) {
            if(aux[i] == '\n') {
                aux[i] = '\0';
                break;
            }
            if(islower(aux[i]))
                aux[i] = toupper(aux[i]);
        }

        if(diccionario_contiene(d, aux)) {
            return strdup(aux);
        }

        printf("ERROR: ¡La palabra no existe en el diccionario!\n");
    }
}

void imprimir_ayuda() {
    for(int i = 0; i < sizeof(tabla_ayudas) / sizeof(tabla_ayudas[0]); i++) {
        ansi_color(tabla_colores[i].frente, tabla_colores[i].fondo);
        putchar('X');
        ansi_reset_color();
        printf(": %s\n", tabla_ayudas[i]);
    }
}

void imprimir_texto(const char *s) {
    printf("%s", s);
}

void imprimir_letras(letra_t * const letras[], size_t n) {
    for(size_t i = 0; i < n; i++) {
        letra_estado_t estado = letra_get_estado(letras[i]);
        ansi_color(tabla_colores[estado].frente, tabla_colores[estado].fondo);
        putchar(letra_get_caracter(letras[i]));
        ansi_reset_color();
        putchar(' ');
    }
    putchar('\n');
}
