/*
main.c Wordle9511, ejemplo de juego sobrediseñado con TDAs.
Copyright (C) 2022 Sebastian Santisi <ssantisi@fi.uba.ar>
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "config.h"
#include "diccionario.h"
#include "interaccion.h"
#include "wordle.h"


#define MAX_CADENA 100


int main(int argc, char *argv[]) {
    diccionario_t *d = diccionario_crear(ARCHIVO_DICCIONARIO);
    if(d == NULL) {
        fprintf(stderr, "Error leyendo diccionario\n");
        return 1;
    }

    char aux[MAX_CADENA];

    int semilla = time(NULL);
    if(argc > 1) {
        semilla = atoi(argv[1]);
    }
    sprintf(aux, "Usando semilla: %d\n", semilla);
    imprimir_texto(aux);
    srand(semilla);

    char *palabra = diccionario_obtener_palabra(d, rand() % diccionario_get_longitud(d));
    if(palabra == NULL) {
        fprintf(stderr, "Error obteniendo palabra\n");
        diccionario_destruir(d);
        return 1;
    }

    wordle_t *w = wordle_crear(palabra);
    if(w == NULL) {
        fprintf(stderr, "Error creando Wordle\n");
        diccionario_destruir(d);
        free(palabra);
        return 1;
    }

    imprimir_texto("Bienvenido al Wordle9511\n\n");
    sprintf(aux, "Tenés %d intentos para acertar una palabra de %d letras\n\n", INTENTOS_MAXIMOS, PALABRA_LONGITUD);
    imprimir_texto(aux);
    imprimir_texto("Convención:\n");
    imprimir_ayuda();

    while(! wordle_esta_ganado(w) && ! wordle_esta_perdido(w)) {
        sprintf(aux, "\nIntento número %zd:\n", wordle_get_cantidad_intentos(w) + 1);
        imprimir_texto(aux);

        wordle_imprimir_alfabeto(w);

        char *intento = leer_palabra(d);
        if(intento == NULL) {
            fprintf(stderr, "Error leyendo intento\n");
            diccionario_destruir(d);
            wordle_destruir(w);
            free(palabra);
            return 1;
        }

        if(! wordle_procesar_intento(w, intento)) {
            fprintf(stderr, "Error procesando intento\n");
            diccionario_destruir(d);
            wordle_destruir(w);
            free(palabra);
            free(intento);
            return 1;
        }

        free(intento);

        wordle_imprimir_intento(w, wordle_get_cantidad_intentos(w) - 1);
    }

    imprimir_texto("\n");
    imprimir_tabla_intentos(w);

    if(wordle_esta_ganado(w)) {
        sprintf(aux, "Felicitaciones, acertaste después de %zd intentos\n", wordle_get_cantidad_intentos(w));
        imprimir_texto(aux);
    }
    else {
        sprintf(aux, "¡Perdiste!, la palabra correcta era %s\n", palabra);
        imprimir_texto(aux);
    }

    diccionario_destruir(d);
    wordle_destruir(w);
    free(palabra);

    return 0;
}

