/*
wordle.h Wordle9511, ejemplo de juego sobrediseñado con TDAs.
Copyright (C) 2022 Sebastian Santisi <ssantisi@fi.uba.ar>
*/
#ifndef WORDLE_H
#define WORDLE_H

#include <stdbool.h>
#include <stddef.h>

// TDA que representa una instancia del juego Wordle
typedef struct wordle wordle_t;

// Crea el el TDA con una palabra a adivinar
wordle_t *wordle_crear(const char *palabra);

// Destruye la memoria asociada a w
void wordle_destruir(wordle_t *w);

// Getter de la cantidad de intentos efectuados sobre w
size_t wordle_get_cantidad_intentos(const wordle_t *w);

// Indica si w ya fue ganado (se adivinó la palabra)
bool wordle_esta_ganado(const wordle_t *w);

// Indica si w ya fue perdido (se superó la cantidad de intentos)
bool wordle_esta_perdido(const wordle_t *w);

// Procesa el intento palabra como solución de w
bool wordle_procesar_intento(wordle_t *w, const char *palabra);

// Imprime la información del intento i de w
bool wordle_imprimir_intento(const wordle_t *w, size_t i);

// Imprime el estado de las letras del alfabeto asociado a w
bool wordle_imprimir_alfabeto(const wordle_t *w);

// Imprime la tabla con el resultado de la partida
bool imprimir_tabla_intentos(const wordle_t *w);

#endif
